Package: scGPS
Type: Package
Title: A complete analysis of single cell subpopulations, from
        identifying subpopulations to analysing their relationship
        (scGPS = single cell Global Predictions of Subpopulation)
Version: 1.24.0
Authors@R: c(
    person("Quan", "Nguyen", email="quan.nguyen@uq.edu.au", 
    role=c("aut", "cre")),
    person("Michael", "Thompson",role=c("aut")),
    person("Anne", "Senabouth",role=c("aut"))
    )
Description: The package implements two main algorithms to answer two key
    questions: a SCORE (Stable Clustering at Optimal REsolution) to find
    subpopulations, followed by scGPS to investigate the relationships between
    subpopulations.
Encoding: UTF-8
LazyData: true
License: GPL-3
BugReports: https://github.com/IMB-Computational-Genomics-Lab/scGPS/issues
url: https://github.com/IMB-Computational-Genomics-Lab/scGPS/
RoxygenNote: 7.1.1
Depends: R (>= 3.6), SummarizedExperiment, dynamicTreeCut,
        SingleCellExperiment
biocViews: SingleCell, Clustering, DataImport, Sequencing, Coverage
Imports: glmnet (> 2.0), caret (>= 6.0), ggplot2 (>= 2.2.1),
        fastcluster, dplyr, Rcpp, RcppArmadillo, RcppParallel,
        grDevices, graphics, stats, utils, DESeq2, locfit
Suggests: Matrix (>= 1.2), testthat, knitr, parallel, rmarkdown,
        RColorBrewer, ReactomePA, clusterProfiler, cowplot,
        org.Hs.eg.db, reshape2, xlsx, dendextend, networkD3, Rtsne,
        BiocParallel, e1071, WGCNA, devtools, DOSE
VignetteBuilder: knitr
LinkingTo: Rcpp, RcppArmadillo, RcppParallel
SystemRequirements: GNU make
git_url: https://git.bioconductor.org/packages/scGPS
git_branch: RELEASE_3_22
git_last_commit: 2fe6e8f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 06:23:28 UTC; biocbuild
Author: Quan Nguyen [aut, cre],
  Michael Thompson [aut],
  Anne Senabouth [aut]
Maintainer: Quan Nguyen <quan.nguyen@uq.edu.au>
