# Generated by roxygen2: do not edit by hand

export(CORE_bagging)
export(CORE_clustering)
export(CORE_subcluster)
export(PCA)
export(PrinComp_cpp)
export(annotate_clusters)
export(bootstrap_parallel)
export(bootstrap_prediction)
export(calcDist)
export(calcDistArma)
export(clustering)
export(clustering_bagging)
export(distvec)
export(find_markers)
export(find_optimal_stability)
export(find_stability)
export(mean_cpp)
export(new_scGPS_object)
export(new_summarized_scGPS_object)
export(plot_CORE)
export(plot_optimal_CORE)
export(plot_reduced)
export(predicting)
export(rand_index)
export(rcpp_Eucl_distance_NotPar)
export(rcpp_parallel_distance)
export(reformat_LASSO)
export(sub_clustering)
export(subset_cpp)
export(summary_accuracy)
export(summary_deviance)
export(summary_prediction_lasso)
export(summary_prediction_lda)
export(tSNE)
export(top_var)
export(tp_cpp)
export(training)
export(var_cpp)
import(DESeq2)
import(RcppArmadillo)
import(RcppParallel)
import(SingleCellExperiment)
import(SummarizedExperiment)
import(caret)
import(dplyr)
import(dynamicTreeCut)
import(ggplot2)
import(glmnet)
import(locfit)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,coef)
importFrom(stats,na.omit)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,sd)
importFrom(utils,capture.output)
importFrom(utils,head)
useDynLib(scGPS)
