#' @name day_5_cardio_cell_sample
#' @title One of the two example single-cell count matrices to be used
#' for \pkg{scGPS} prediction
#' @description The count data set contains counts for 17402 genes for 983 cells
#' (1 row per gene) randomly subsampled from day-5 cardio-differentiation 
#' population
#' The vector of clustering information contains corresponding to cells in the
#' count matrix
#' @return a list, with the name day5
#' @docType data
#' @usage day_5_cardio_cell_sample
#' @format a list instance, containing a count matrix and a vector with 
#' clustering information.
#' @source Dr Joseph Powell's laboratory, IMB, UQ
#' @author Quan Nguyen, 2017-11-25
"day_5_cardio_cell_sample"
