#' @title One of the two example single-cell count matrices to be used
#' for training \pkg{scGPS} model
#' @description The count data set contains counts for 16990 genes for 590 cells
#' randomly subsampled from day-2 cardio-differentiation population.
#' The vector of clustering information contains corresponding to cells in the 
#' count matrix
#' @return a list, with the name day2
#' @docType data
#' @usage day_2_cardio_cell_sample
#' @format a list instance, containing a count matrix and a vector with 
#' clustering information
#' @source Dr Joseph Powell's laboratory, IMB, UQ
#' @author Quan Nguyen, 2017-11-25
"day_2_cardio_cell_sample"
