#' @name training_gene_sample
#' @title Input gene list for training \pkg{scGPS}, e.g. differentially 
#' expressed genes
#' @description Genes should be ordered from most to least important genes
#' (1 row per gene)
#' @return a vector containing gene symbols
#' @docType data
#' @usage training_gene_sample
#' @format a list instance, containing a count matrix and a vector with 
#' clustering information.
#' @source Dr Joseph Powell's laboratory, IMB, UQ
#' @author Quan Nguyen, 2017-11-25
"training_gene_sample"
