% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{mean_cpp}
\alias{mean_cpp}
\title{Calculate mean}
\usage{
mean_cpp(x)
}
\arguments{
\item{x}{integer.}
}
\value{
a scalar value
}
\description{
Calculate mean
}
\examples{
mean_cpp(seq_len(10^6))

}
